//+------------------------------------------------------------------+
//|                                                     mp order.mq4 |
//|                                      Copyright  2008, sptrading | 
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, sptrading"
#property link      "http://www.sptrading.net/"

#include <mplib.mqh>
#include <sptlib.mqh>

//+------------------------------------------------------------------+
//| mode: mode selection                                             |
//|     1: responsive long                                           |
//|     2: responsive short                                          |
//|     3: initiating long                                           |
//|     4: initiating short                                          |
//+------------------------------------------------------------------+

extern int mode = 1;
extern bool daily = false;
extern string filename = "";
extern bool AsiaSession = true;
extern bool EUSession = true;
extern bool USASession = true;

//+------------------------------------------------------------------+
//| Profile by date                              |
//+------------------------------------------------------------------+
void mpoMarketProfileByDate(double& pocsPrices[], datetime dt[], double onetick, int sessions, bool drawText, bool drawBox, bool drawPOC)
{  
   int shifts[];
   int size = ArraySize(dt);
   int r = ArrayResize(shifts, size);
   
   // Sort and find bar shifts
   int n = ArraySort(dt, WHOLE_ARRAY, 0, MODE_DESCEND);
   for (int i = 0; i < size; i++)
   {
      shifts[i] = iBarShift(NULL, 0, dt[i]);
      //Alert(i, ", ", shifts[i], ", ", TimeToStr(dt[i]));
   }
 
   // first profile is the current
   if (size > 0)
   {
      //Alert(i, ", ", TimeToStr(dt[i]), ", ", shifts[i]);
      int beginBar = 0;
      int bars = shifts[0]-beginBar;
      mpMarketProfile(pocsPrices, beginBar, bars, onetick, sessions, drawText, drawBox, drawPOC);
   }
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
void mpoMarketProfileByDay(double& pocsPrices[], int days, double onetick, int sessions, bool drawText, bool drawBox, bool drawPOC)
{  
   int beginBar = 0;
   int bars = 0;
   int count = 0;
   for (int i = 0; i < days; i++)
   {
      // Find Bars
      while (TimeDayOfYear(Time[count]) == TimeDayOfYear(Time[count+1]))
      {
         count += 1;
         bars += 1;
         //if (bars > 60*24)
         //   break;
      }
      //Alert(beginBar, ", ", bars, ", ", count, ", ", TimeToStr(Time[count]));
      //if (TimeDayOfWeek(Time[count]) > 0)
      if (i==0)
      {
         mpMarketProfile(pocsPrices, beginBar, bars, onetick, sessions, drawText, drawBox, drawPOC);
      }
      else
      {
         double pocsPc[];
         mpMarketProfile(pocsPc, beginBar, bars, onetick, sessions, drawText, drawBox, drawPOC);
      }
      beginBar = count;
      bars = 0;
      count += 1;
   }
}

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//----
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+

int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
   //----
   int cnt, total;
   double lots = 0.01;
   int daytime = 24*60*60;
   int mpOmb = 3434;

   // Read file
   if (filename == "")
      filename = mpNameFromPeriod() + ".mp";
 
   datetime dt[];
   double pocsPrices[];
   // Sessions  
   int sessions = 0;
   if (AsiaSession)
     sessions |= mpeAsiaSession;
   if (EUSession)
     sessions |= mpeEUSession;
   if (USASession)
     sessions |= mpeUSASession;
   double onetick = 1/(MathPow(10,Digits));  
   if (daily)
   {
      //mpoMarketProfileByDay(pocsPrices, days, onetick, sessions, false, false, false);
   }
   else
   {
      bool success = mpReadDatetime(dt, filename);
      if (success)
        mpoMarketProfileByDate(pocsPrices, dt, onetick, sessions, false, false, false);
   }   
   
   total=OrdersTotal();
   //barShift = iBarShift(NULL, int timeframe, datetime time, bool exact=false) 

   int mpTotal = 0;
   for(cnt=0;cnt<total;cnt++)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      int omb = OrderMagicNumber();
      string symbol = OrderSymbol() ;

      if (omb == mpOmb && symbol == Symbol())
         mpTotal += 1;
   }
   
   if(mpTotal<1) 
   {
      int ticketLong = -1;
      int ticketShort = -1;
      double orderTriggerPrice;
      double buyPrice;
      double sellPrice;
      double shortPrice;
      double coverPrice;
      double targetPrice;
      datetime expiration = Time[0] + daytime;
      bool filter = false;

      // no opened orders identified
      switch (mode)
      {
         // responsive long
         case 1:
            orderTriggerPrice = NormalizeDouble(pocsPrices[mpeStd2Down],Digits);
            buyPrice = NormalizeDouble(pocsPrices[mpeStd1Down],Digits);
            sellPrice = NormalizeDouble(pocsPrices[mpeStd2Down],Digits);
            targetPrice = NormalizeDouble(pocsPrices[mpeStd1Up],Digits);
            filter = Ask < orderTriggerPrice;
            Print(Symbol(), ", ", mode, ", Trg: ", orderTriggerPrice, ", Buy@", buyPrice, ", StopLoss@", sellPrice, ", Target@", targetPrice, ", go: ", filter);
            break;
            
         // responsive short
         case 2:
            orderTriggerPrice = NormalizeDouble(pocsPrices[mpeStd2Up],Digits);
            shortPrice = NormalizeDouble(pocsPrices[mpeStd1Up],Digits);
            coverPrice = NormalizeDouble(pocsPrices[mpeStd2Up],Digits);
            targetPrice = NormalizeDouble(pocsPrices[mpeStd1Down],Digits);
            filter = Bid > orderTriggerPrice;
            Print(Symbol(), ", ", mode, ", Trg: ", orderTriggerPrice, ", Short@", shortPrice, ", Cover@", coverPrice, ", Target@", targetPrice, ", go: ", filter);
            break;

         // initiating long
         case 3:
            orderTriggerPrice = NormalizeDouble(pocsPrices[mpeUnitUp],Digits);
            buyPrice = NormalizeDouble(pocsPrices[mpeUnitUp]+(pocsPrices[mpeUnitUp]-pocsPrices[mpeStd2Up]),Digits);
            sellPrice = NormalizeDouble(pocsPrices[mpePOC],Digits);
            targetPrice = NormalizeDouble(pocsPrices[mpeUnitUp]+2*(pocsPrices[mpeUnitUp]-pocsPrices[mpeStd2Up]),Digits);
            filter = Ask > orderTriggerPrice;
            Print(Symbol(), ", ", mode, ", Trg: ", orderTriggerPrice, ", Buy@", buyPrice, ", StopLoss@", sellPrice, ", Target@", targetPrice, ", go: ", filter);
            break;
            
         // initiating short
         case 4:
            orderTriggerPrice = NormalizeDouble(pocsPrices[mpeUnitDown],Digits);
            shortPrice = NormalizeDouble(pocsPrices[mpeUnitDown]-(pocsPrices[mpeStd2Down]-pocsPrices[mpeUnitDown]),Digits);
            coverPrice = NormalizeDouble(pocsPrices[mpePOC],Digits);
            targetPrice = NormalizeDouble(pocsPrices[mpeUnitDown]-2*(pocsPrices[mpeStd2Down]-pocsPrices[mpeUnitDown]),Digits);
            filter = Ask < orderTriggerPrice;
            Print(Symbol(), ", ", mode, ", Trg: ", orderTriggerPrice, ", Short@", shortPrice, ", Cover@", coverPrice, ", Target@", targetPrice, ", go: ", filter);
            break;
      }
      
      if (filter)
      {
         switch (mode)
         {
         // responsive long
         case 1:
            Print(TimeToStr(TimeCurrent()), ", RBuy@", buyPrice, "-", sellPrice, ", ", targetPrice);
            ticketLong=OrderSend(Symbol(),OP_BUYSTOP,lots,buyPrice,3,sellPrice,targetPrice,"mp responsive long",mpOmb,expiration,Green);
            break;
            
         // responsive short
         case 2:
            Print(TimeToStr(TimeCurrent()), ", RSell@", shortPrice, "-", coverPrice, ", ", targetPrice);
            ticketShort=OrderSend(Symbol(),OP_SELLSTOP,lots,shortPrice,3,coverPrice,targetPrice,"mp responsive Short",mpOmb,expiration,Green);
            break;

         // initiating long
         case 3:
            Print(TimeToStr(TimeCurrent()), ", RBuy@", buyPrice, "-", sellPrice, ", ", targetPrice);
            ticketLong=OrderSend(Symbol(),OP_BUYSTOP,lots,buyPrice,3,sellPrice,targetPrice,"mp initiating long",mpOmb,expiration,Green);
            break;
            
         // initiating short
         case 4:
            Print(TimeToStr(TimeCurrent()), ", RSell@", shortPrice, "-", coverPrice, ", ", targetPrice);
            ticketShort=OrderSend(Symbol(),OP_SELLSTOP,lots,shortPrice,3,coverPrice,targetPrice,"mp initiating Short",mpOmb,expiration,Green);
            break;
         }
      }

      if(ticketLong>=0)
      {
         if(OrderSelect(ticketLong,SELECT_BY_TICKET,MODE_TRADES)) 
         {
            spOrderOpenSendMail(ticketLong, OP_BUYSTOP,lots,buyPrice, sellPrice, targetPrice, "mp long",expiration);
            Print("mp long order opened : ",OrderOpenPrice());
         }
         else
         {
            Print("Error opening mp long order : ",GetLastError(), TimeToStr(TimeCurrent()), ", ticker: ", ticketLong);
         }  
         return(0); 
      }

      if(ticketShort>=0)
      {
         if(OrderSelect(ticketShort,SELECT_BY_TICKET,MODE_TRADES)) 
         {
            spOrderOpenSendMail(ticketShort, OP_SELLSTOP,lots,sellPrice, coverPrice, targetPrice, "mp Short",expiration);
            Print("mp Short order opened : ",OrderOpenPrice());
         }
         else
         {
            Print("Error opening mp short order : ",GetLastError(), TimeToStr(TimeCurrent()), ", ticker: ", ticketShort);
         }  
         return(0); 
      }
   }
   else
   {
      datetime oot = OrderOpenTime();
      datetime tc = TimeCurrent();
      bool isNextDay = (TimeDay(tc) > TimeDay(oot)) || (TimeMonth(tc) > TimeMonth(oot)) ||(TimeYear(tc) > TimeYear(oot));
      int ticket = -1;

      //Print("Diff: ", TimeCurrent(), ", ", expiration, ", ", TimeCurrent()-expiration);
      // Select this ticker mp order
      for(cnt=0;cnt<total;cnt++)
      {
         OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
         omb = OrderMagicNumber();
         symbol = OrderSymbol() ;

         if (omb == mpOmb && symbol == Symbol())
         {
            ticket = OrderTicket();      
            break;
         }
      }

      // 0. cancel if not executed one day after order send
      if(isNextDay && OrderType()==OP_BUYSTOP && ticket >= 0)
      {
         OrderDelete(ticket);
         Print("mp long order deleted : ",ticket);
         return(0); 
      }
      if(isNextDay && OrderType()==OP_SELLSTOP && ticket >= 0)
      {
         OrderDelete(ticket);
         return(0);  
      }    
  }

//----
   return(0);
}

