//+------------------------------------------------------------------+
//|                                                        mplib.mqh |
//|                                      Copyright  2008, sptrading | 
//+------------------------------------------------------------------+
#import "mplib.ex4"

/**
 Purpose: session
*/
static int mpeAsiaSession = 1;
static int mpeEUSession = 2;
static int mpeUSASession = 4;
static int mpeAllSessions = 7;

/**
 Purpose: draw a single market profile
 Params: bars, tick, draw options
 Return: poc prices
*/
void mpMarketProfile(double& pocsPrices[], int beginBar, int bars, double onetick, int sessions, bool drawText = true, bool drawBox = true, bool drawPOC = true);

/**
 Purpose: draw multiple market profiles
 Params: days, datetime or file with datetime
 Return: poc prices of the newest profile
*/
void mpMarketProfileByDay(double& pocsPrices[], int days, double onetick, int sessions, bool drawText = true, bool drawBox = true, bool drawPOC = true);
void mpMarketProfileByDate(double& pocsPrices[], datetime dt[], double onetick, int sessions, bool drawText = true, bool drawBox = true, bool drawPOC = true);
void mpMarketProfileByFile(double& pocsPrices[], string filename, double onetick, int sessions, bool drawText = true, bool drawBox = true, bool drawPOC = true);


/**
 Purpose: draw a single market profile
 Params: bars, tick, draw options
 Return: poc prices
*/
void mpSendMailAlert(double pocsPrices[], bool std1Breakout, bool unitBreakout);

/**
 Purpose: fill the price levels
 Params: begin bar, number of bars, tick value
 Return: prices updated
 Note: price is resized
*/
void mpPrices(double& prices[], int beginBar, int bars, double onetick, int sessions);

/**
 Purpose: fill the mp datetime (string formatted)
 Params: prices, begin bar, number of bars (same as used in mpPrices())
 Return: mp datetime updated
 Note: mp is resized
*/
void mpDatetime(string& mpDt[], double prices[], int beginBar, int bars, int sessions);

/**
 Purpose: fill a line datetime array from a string formatted datetime
 Params: mpDt in datetime format
 Return: dt updated
 Note: dt is resized
*/
void mpMarketDtStrToDatetime(datetime& dt[], string mpDt);

/**
 Purpose: fill the mp distribution in text mode
 Params: mp in datetime format
 Return: mpText updated
 Note: mpText is resized
*/
string mpMarketProfileTextLine(string mp);
void mpMarketProfileText(string& mpText[], string& mpDt[]);

/**
 Purpose: find POC and value area indexes
 Params: datetime
 Return: index
 Note: use these vars to access pocsIndex
*/
static int mpePOC = 0;
static int mpeStd1Down = 1;
static int mpeStd1Up = 2;
static int mpeStd2Down = 3;
static int mpeStd2Up = 4;
static int mpeUnitDown = 5;
static int mpeUnitUp = 6;
static int mpeExcessDown = 7;
static int mpeExcessUp = 8;
static int mpeNumbers = 9;
void mpPocIndex(int& pocsIndex[], string mpDt[]);

/**
 Purpose: find POC prices
 Params: prices, index
 Return: prices
 Note: resize pocPrices, use these vars to access pocPrices
*/
void mpPocPrices(double& pocsPrices[], double prices[], int pocsIndex[]);

/**
 Purpose: read datetime from files
 Params: filename
 Return: datetime, true if success
 Note: resize dt
*/
bool mpReadDatetime(datetime& dt[], string filename);

/**
 Purpose: bars shifts from datetime
 Params: datetime
 Return: shifts
 Note: resize shifts
*/
void mpBarShiftsFromDatetime(int& shifts[], datetime dt[]);

/**
 Purpose: log prices to file
 Params: price, filename, bars
 Return: true if success
*/
bool mpLogPocPrices(double pocsPrices[], string filename, int beginBar, int bars, int sessions = 7);
bool mpMarketProfileLog(string datetimeFilename, string pocFilename, int sessions = 7);

/**
 Purpose: human name from period
 Return: name (as M5 for 5 mins, D1 1 day...)
*/
string mpNameFromPeriod();

/********************************************
 Purpose: drawing utilities
********************************************/

/**
 Purpose: draw mp distribution in text mode
 Params: prices, etxt, begin bar, number of bars, tick size (same as used in mpPrices())
 Return: none
*/
void mpDrawText(double prices[], string mpText[], int beginBar, int bars, double onetick, color col = White);
void mpDrawBox(double prices[], string mpText[], int beginBar, int bars, double onetick);

/**
 Purpose: draw POCS rices
 Params: POC prices, bars
 Return: none
*/
void mpDrawPocs(double& pocsPrices[], int beginBar, int bars);

/**
 Purpose: erase all drawings
 Return: none
*/
void mpDrawErase();

/********************************************
 Purpose: utilities
********************************************/

/**
 Purpose: format a string left justified
 Params: string to fotmat
 Return: string fotmatted
*/
string mpFormatLeft(string text);

/**
 Purpose: find the appropriate mp symbol at that time
 Params: time
 Return: mp symbol
*/
string mpSymbolTime(datetime dt);

/**
 Purpose: find the appropriate mp color for that mp code or at that time
 Params: mp code or time
 Return: mp color
*/
color mpSymbolColorByCode(string text);
color mpSymbolColorByTime(datetime dt);


